@@ -28,7 +28,7 @@ format_trans <- function(x) {
   direc_df <- data.frame(
     direction = c(forw, back),
     names = c(rep("forward", 2), rep("backward", 2)),
-    stringsAsFactors = F
+    stringsAsFactors = FALSE
   )
 
   direct <- direc_df$names[match(x[[1]], direc_df$direction)]
